@@ -1,4 +1,4 @@
 #define OP MFB_OP_BLACK
 #define MFBTEGLYPHBLT mfbTEGlyphBltBlack
 #define CLIPTETEXT mfbImageGlyphBltBlack
-#include "../../mfb/mfbtegblt.c"
+#include "./mfbtegblt.c"
